<!DOCTYPE html>
<html lang="{{request.locale_name}}">
  <head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <meta name="description" content="pyramid web application">
    <meta name="author" content="Pylons Project">
    <link rel="shortcut icon" href="{{request.static_url('neuralknight:static/pyramid-16x16.png')}}">

    <title>neuralknight</title>

    <!-- Bootstrap core CSS -->
    <link href="//oss.maxcdn.com/libs/twitter-bootstrap/3.0.3/css/bootstrap.min.css" rel="stylesheet">

    <!-- Custom styles for this scaffold -->
    <link href="{{request.static_url('neuralknight:static/reset.css')}}" rel="stylesheet">
    <link href="{{request.static_url('neuralknight:static/style.css')}}" rel="stylesheet">
  <link href="https://fonts.googleapis.com/css?family=Share+Tech" rel="stylesheet">
    <!-- HTML5 shim and Respond.js IE8 support of HTML5 elements and media queries -->
    <!--[if lt IE 9]>
      <script src="//oss.maxcdn.com/libs/html5shiv/3.7.0/html5shiv.js" integrity="sha384-0s5Pv64cNZJieYFkXYOTId2HMA2Lfb6q2nAcx2n0RTLUnCAoTTsS0nKEO27XyKcY" crossorigin="anonymous"></script>
      <script src="//oss.maxcdn.com/libs/respond.js/1.3.0/respond.min.js" integrity="sha384-f1r2UzjsxZ9T4V1f2zBO/evUqSEOpeaUUZcMTz1Up63bl4ruYnFYeM+BxI4NhyI0" crossorigin="anonymous"></script>
    <![endif]-->
  </head>

  <body>
    <h1>neuralknight</h1>
    <p>An occasionally suicidal chess AI</p>
    <p><a href="https://github.com/dsnowb/neuralknight">Source on Github</a></p>
    <p>Written by:</p>
    <ul>
      <li><a href=https://github.com/dsnowb>David Snowberger</a></li>
      <li><a href=https://github.com/shannontully>Shannon Tully</a></li>
      <li><a href=https://github.com/agrandquist>Adam Grandquist</a></li>
    </ul>
  </body>
</html>
